using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows.Input;
using Microsoft.Maui.Controls;
using SharedLib;

namespace BindingTestApp;

public class MainPageViewModel : INotifyPropertyChanged
{
    public MainPageViewModel(ITestClass testClass)
    {
        Message = "N/A";
        DoSomethingCommand = new Command(() => DoSomething(testClass));
    }

    private string _message;

    public string Message
    {
        get => _message;
        set => SetField(ref _message, value);
    }
    
    public ICommand DoSomethingCommand { get; }
    
    private void DoSomething(ITestClass tc)
    {
        try
        {
            tc.DoSomething();
            Message = "Successful";
        }
        catch (Exception e)
        {
            Message = $"ERROR: {e.GetType().FullName} / {e.Message}";
        }
    }

    public event PropertyChangedEventHandler? PropertyChanged;

    protected virtual void OnPropertyChanged([CallerMemberName] string? propertyName = null)
    {
        PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
    }

    protected bool SetField<T>(ref T field, T value, [CallerMemberName] string? propertyName = null)
    {
        if (EqualityComparer<T>.Default.Equals(field, value)) return false;
        field = value;
        OnPropertyChanged(propertyName);
        return true;
    }
}